@@ -149,6 +149,7 @@ def get_bam_stats(params, ref_lengths=None, scale=1e6):
             min_mapping_quality=0,
         )
     ]
+    samfile.close()
     # convert datafrane to pyranges
     ranges = create_pyranges(reference, starts, ends, strands)
     ranges = ranges.merge(strand=False).lengths().to_list()
@@ -323,7 +324,7 @@ class BamAlignment:
             read_length_std = np.std(self.read_length, ddof=1)
             read_length_max = np.max(self.read_length)
             read_length_min = np.min(self.read_length)
-            read_length_mode = stats.mode(self.read_length)[0][0]
+            read_length_mode = stats.mode(self.read_length, keepdims=True).mode[0]
             read_aligned_length = np.mean(self.read_aligned_length)
             read_aln_score = np.mean(self.read_aln_score)
             mapping_quality = np.mean(self.mapping_quality)
@@ -378,7 +379,7 @@ class BamAlignment:
 
 
 # Inspired from https://gigabaseorgigabyte.wordpress.com/2017/04/14/getting-the-edit-distance-from-a-bam-alignment-a-journey/
-def process_bam(bam, threads=1, reference_lengths=None, scale=1e6):
+def process_bam(bam, threads=1, reference_lengths=None, scale=1e6, min_read_count=0):
     """
     Processing function: calls pool of worker functions
     to extract from a bam file two definitions of the edit distances to the reference genome scaled by read length
@@ -425,27 +426,25 @@ def process_bam(bam, threads=1, reference_lengths=None, scale=1e6):
     total_refs = samfile.nreferences
     logging.info(f"Found {total_refs:,} reference sequences")
     # logging.info(f"Found {samfile.mapped:,} alignments")
-    logging.info(f"Removing references without mappings...")
-    # Remove references without mapped reads
-    alns_in_ref = defaultdict(int)
-    for aln in tqdm.tqdm(
-        samfile.fetch(until_eof=True),
-        total=samfile.mapped,
-        leave=False,
-        ncols=80,
-        desc=f"Alignments processed",
-    ):
-        alns_in_ref[aln.reference_name] += 1
 
-    references = [ref for ref, count in alns_in_ref.items() if count > 0]
+    logging.info(
+        f"Removing references without mappings or less than {min_read_count} reads..."
+    )
+
+    references = [
+        chrom.contig
+        for chrom in samfile.get_index_statistics()
+        if chrom.mapped > min_read_count
+    ]
 
     if len(references) == 0:
         logging.error("No reference sequences with alignments found in the BAM file")
         sys.exit(1)
 
     logging.info(f"Keeping {len(references):,} references")
-
+    
     params = zip([bam] * len(references), references)
+
     try:
         logging.info(f"Getting stats for each reference...")
 
@@ -513,7 +512,7 @@ def filter_reference_BAM(
         filter_conditions (dict): A dictionary with the filter conditions to be used
         out_files (dict): Where to save the BAM files.
     """
-    logging.info(f"Filtering stats...")
+    logging.info("Filtering stats...")
     logging.info(
         f"min_read_count >= {filter_conditions['min_read_count']} & min_read_length >= {filter_conditions['min_read_length']} & min_read_ani >= {filter_conditions['min_read_ani']} & min_expected_breadth_ratio >= {filter_conditions['min_expected_breadth_ratio']} &  min_breadth >= {filter_conditions['min_breadth']} & min_coverage_evenness >= {filter_conditions['min_coverage_evenness']}"
     )
@@ -531,9 +530,9 @@ def filter_reference_BAM(
             out_files["stats_filtered"], sep="\t", index=False, compression="gzip"
         )
         if only_stats_filtered:
-            logging.info(f"Skipping saving filtered BAM file.")
+            logging.info("Skipping saving filtered BAM file.")
         else:
-            logging.info(f"Writing filtered BAM file... (be patient)")
+            logging.info("Writing filtered BAM file... (be patient)")
             refs_dict = dict(
                 zip(df_filtered["reference"], df_filtered["reference_length"])
             )
@@ -550,40 +549,23 @@ def filter_reference_BAM(
                 list(ref_names), list(ref_lengths)
             )
             references = df_filtered["reference"].values
-            params = zip([bam] * len(references), references)
-            try:
-                logging.info(f"Filtering BAM file...")
 
-                if is_debug():
-                    alns = list(map(get_alns, params))
-                else:
-
-                    p = Pool(threads)
-                    c_size = calc_chunksize(threads, len(references))
-                    alns = list(
-                        tqdm.tqdm(
-                            p.imap_unordered(get_alns, params, chunksize=c_size),
-                            total=len(references),
-                            leave=False,
-                            ncols=80,
-                            desc=f"References processed",
-                        )
+            logging.info("Filtering BAM file...")
+            samfile = pysam.AlignmentFile(bam, "rb")
+            for reference in tqdm.tqdm(
+                references,
+                total=len(references),
+                leave=False,
+                ncols=80,
+                desc="References processed",
+            ):
+                for aln in samfile.fetch(reference=reference, multiple_iterators=False):
+                    out_bam_file.write(
+                        pysam.AlignedSegment.fromstring(aln.to_string(), header=header)
                     )
-
-                    p.close()
-                    p.join()
-
-            except KeyboardInterrupt:
-                logging.info(f"User canceled the operation. Terminating jobs.")
-                p.terminate()
-                p.join()
-                sys.exit(0)
-
-            for aln in fast_flatten(alns):
-                out_bam_file.write(pysam.AlignedSegment.fromstring(aln, header=header))
             out_bam_file.close()
             if sort_by_name:
-                logging.info(f"Sorting BAM file by read name...")
+                logging.info("Sorting BAM file by read name...")
                 pysam.sort(
                     "-n",
                     "-@",
@@ -614,16 +596,16 @@ def filter_reference_BAM(
                 pysam.set_verbosity(save)
                 samfile.close()
 
-                logging.info(f"BAM index not found. Indexing...")
+                logging.info("BAM index not found. Indexing...")
                 if max_chr_length > 536870912:
-                    logging.info(f"A reference is longer than 2^29, indexing with csi")
+                    logging.info("A reference is longer than 2^29, indexing with csi")
                     pysam.index(out_files["bam_filtered"], "-c")
                 else:
                     pysam.index(out_files["bam_filtered"])
 
             os.remove(out_files["bam_filtered_tmp"])
     else:
-        logging.info(f"No references meet the filter conditions. Skipping...")
+        logging.info("No references meet the filter conditions. Skipping...")
 
 
 def get_alns(params):
